<?php
/*
* A payment plugin called "coinpayments". This is the main file of the plugin.
*/

// You need to extend from the hikashopPaymentPlugin class which already define lots of functions in order to simplify your work
class plgHikashoppaymentCoinpayments extends hikashopPaymentPlugin
{
	var $accepted_currencies = array(); //List of the plugin's accepted currencies. The plugin won't appear on the checkout if the current currency is not in that list. You can remove that attribute if you want your payment plugin to display for all the currencies
	var $multiple = true; // Multiple plugin configurations. It should usually be set to true
	var $name = 'coinpayments'; //Payment plugin name (the name of the PHP file)
	
	var $cancel_url;
	var $success_url;
	var $ipn_url;

	// This array contains the specific configuration needed (Back end > payment plugin edition), depending of the plugin requirements.
	// They will vary based on your needs for the integration with your payment gateway.
	// The first parameter is the name of the field. In upper case for a translation key.
	// The available types (second parameter) are: input (an input field), html (when you want to display some custom HTML to the shop owner), textarea (when you want the shop owner to write a bit more than in an input field), big-textarea (when you want the shop owner to write a lot more than in an input field), boolean (for a yes/no choice), checkbox (for checkbox selection), list (for dropdown selection) , orderstatus (to be able to select between the available order statuses)
	// The third parameter is the default value.
	var $pluginConfig = array(
		'merchant_id' => array("Merchant ID",'input'), //User's identifier on the payment platform
		'ipn_secret' => array("IPN Secret",'input'), //User's password on the payment platform
		'debug_email' => array('Debug Email (optional, recommended while testing)', 'input'), //Write some things on the debug file
		'invalid_status' => array('INVALID_STATUS', 'orderstatus'), //Invalid status for order in case of problem during the payment process
		'verified_status' => array('VERIFIED_STATUS', 'orderstatus'), //Valid status for order if the payment has been done well
		'collect_shipping' => array('Send shipping info to CoinPayments', 'boolean', '1'),
		'allow_zero_confirm' => array('Enable zero-confirmation payments (recommended for digital downloads only)', 'boolean', '0'),
	);

	// The constructor is optional if you don't need to initialize some parameters of some fields of the configuration and not that it can also be done in the getPaymentDefaultValues function as you will see later on
	function __construct(&$subject, $config)
	{
		$this->cancel_url = HIKASHOP_LIVE."index.php?option=com_hikashop&ctrl=order&task=cancel_order";
		$this->success_url = HIKASHOP_LIVE."index.php?option=com_hikashop&ctrl=checkout&task=after_end";
		$this->ipn_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&amp;notif_payment=coinpayments&tmpl=component';
		return parent::__construct($subject, $config);
	}


	//This function is called at the end of the checkout. That's the function which should display your payment gateway redirection form with the data from HikaShop
	function onAfterOrderConfirm(&$order,&$methods,$method_id)
	{
		parent::onAfterOrderConfirm($order,$methods,$method_id); // This is a mandatory line in order to initialize the attributes of the payment method

		//Here we can do some checks on the options of the payment method and make sure that every required parameter is set and otherwise display an error message to the user
		if (empty($this->payment_params->merchant_id)) //The plugin can only work if those parameters are configured on the website's backend
		{
			$this->app->enqueueMessage('You have to enter your CoinPayments Merchant ID first : check your plugin\'s parameters, on your website backend','error');
			//Enqueued messages will appear to the user, as Joomla's error messages
			return false;
		}
		elseif (empty($this->payment_params->ipn_secret))
		{
			$this->app->enqueueMessage('You have to enter your CoinPayments IPN Secret first : check your plugin\'s parameters, on your website backend','error');
			return false;
		}
		else
		{
			// Patch - JT 07/18/19 Updated vars fields item_name, amountf, email(DEP)
			$vars = array(
				'cmd' => '_pay',
				'reset' => 1,
				'merchant' => $this->payment_params->merchant_id,
				'item_name' => "Order ID".$order->order_number,
				'custom' => $order->order_id,
				'amountf' => round($order->cart->full_total->prices[0]->price_value_with_tax, 8),
				'currency' => $this->currency->currency_code,
				'success_url' => $this->success_url,
				'cancel_url' => $this->cancel_url,
				'ipn_url' => $this->ipn_url,
				'first_name' => $order->cart->shipping_address->address_firstname,
				'last_name' => $order->cart->shipping_address->address_lastname,
			);
			if ($this->payment_params->collect_shipping == "1") {
				$vars = array_merge($vars, array(
                			'address1' => $order->cart->shipping_address->address_street,
					'address2' => $order->cart->shipping_address->address_street2,
					'city' => $order->cart->shipping_address->address_city,
					'state' => $order->cart->shipping_address->address_state->zone_code_3,
					'zip' => $order->cart->shipping_address->address_post_code,
					'country' => $order->cart->shipping_address->address_country->zone_code_3,
					'phone' => $order->cart->shipping_address->address_telephone,
				));
			}

			//Hash generated to certify the values integrity
			//This hash is generated according to the plateform requirements
			$this->vars = $vars;

			//Ending the checkout, ready to be redirect to the plateform payment final form
			//The showPage function will call the coinpayments_end.php file which will display the redirection form containing all the parameters for the payment platform
			return $this->showPage('end'); 
		}
	}


	//To set the specific configuration (back end) default values (see $pluginConfig array)
	function getPaymentDefaultValues(&$element)
	{
		$element->payment_name='CoinPayments';
		$element->payment_description='You can pay by Bitcoin or other cryptocurrencies via CoinPayments';
		$element->payment_images='CoinPayments';
		$element->payment_params->collect_shipping=1;
		$element->payment_params->address_type="shipping";
		$element->payment_params->notification=1;
		$element->payment_params->invalid_status='cancelled';
		$element->payment_params->verified_status='confirmed';
	}
	
	function errorAndDie($error_msg) {
		if (!empty($this->payment_params->debug_email)) {
			$report = "Error Message: ".$error_msg."\n\n";
			$report .= "POST Fields\n\n";
			foreach ($_POST as $key => $value) {
				$report .= $key.'='.$value."\n";
			}
			
			$report .= "\nAUTH User Passed: |".$_SERVER['PHP_AUTH_USER']."|\n";
			$report .= "AUTH User Expected: |".trim($this->merchant_id)."|\n\n";
			$report .= "AUTH Pass Passed: |".$_SERVER['PHP_AUTH_PW']."|\n\n";
			$report .= "AUTH Pass Expected: |".trim($this->ipn_secret)."|\n\n";			
			
			mail($this->payment_params->debug_email, "CoinPayments.net Invalid IPN", $report);
		}
		die('IPN Error: '.$error_msg);
	}
		
	function authorize_coinpayments_ipn($posted) {
		$error_msg = "Unknown error";
		
		var_dump($this->payment_params);
		var_dump($posted);
		
		if (isset($posted['ipn_mode']) && $posted['ipn_mode'] == 'hmac') {
			if (isset($_SERVER['HTTP_HMAC']) && !empty($_SERVER['HTTP_HMAC'])) {
				$request = file_get_contents('php://input');
				if ($request !== FALSE && !empty($request)) {
					if (isset($posted['merchant']) && $posted['merchant'] == trim($this->payment_params->merchant_id)) {
						$hmac = hash_hmac("sha512", $request, trim($this->payment_params->ipn_secret));
						if ($hmac == $_SERVER['HTTP_HMAC']) {
							return true;
						} else {
							$error_msg = 'HMAC signature does not match';
						}
					} else {
						$error_msg = 'No or incorrect Merchant ID passed';
					}
				} else {
					$error_msg = 'Error reading POST data';
				}
			} else {
				$error_msg = 'No HMAC signature sent.';
			}
		} else {
			if (isset($_SERVER['PHP_AUTH_USER']) && isset($_SERVER['PHP_AUTH_PW']) && $_SERVER['PHP_AUTH_USER'] == trim($this->payment_params->merchant_id) && $_SERVER['PHP_AUTH_PW'] == trim($this->payment_params->ipn_secret)) {
				return true;
			} else {
				$error_msg = "Invalid merchant id/ipn secret";
			}
		}
		$this->errorAndDie($error_msg);
	}

	//After submiting the plateform payment form, this is where the website will receive the response information from the payment gateway servers and then validate or not the order
	function onPaymentNotification(&$statuses)
	{
		//We first create a filtered array from the parameters received
		$posted = array();
		$filter = JFilterInput::getInstance();
		foreach($_REQUEST as $key => $value)
		{
			$key = $filter->clean($key);
			$value = JRequest::getString($key);
			$posted[$key]=$value;
		}
		
		if (!isset($posted['custom'])) {
			die("No 'custom' field passed!\n");
		}
		
		//We load the parameters of the plugin in $this->payment_params and the order data based on the order_id coming from the payment platform
		$order_id = (int)$posted['custom'];
		$dbOrder = $this->getOrder($order_id);
		$this->loadPaymentParams($dbOrder);
		if(empty($this->payment_params))
			die("Error loading payment params!");
		$this->loadOrderData($dbOrder);

		$this->authorize_coinpayments_ipn($posted);

		if ($posted['ipn_type'] != "button") {
			$this->errorAndDie('Invalid IPN type!');
		}
		if ($posted['merchant'] != $this->payment_params->merchant_id) {
			$this->errorAndDie('Invalid merchant ID!');
		}
		if ($posted['currency1'] != $this->currency->currency_code) {
			$this->errorAndDie('Invalid original currency!');
		}
		
		if (floatval($posted['amount1']) < round($dbOrder->order_full_price, 8)) {
			$this->errorAndDie('Payment amount is less than order total!');
		}
		
		$status = intval($posted['status']);
		print "CoinPayments transaction status: ".$status.": ".$posted['status_text']."\n";
		$complete = false;
		if ($status < 0) {
			$this->modifyOrder($order_id, $this->payment_params->invalid_status, true, true); 
		} else if ($status >= 100 || $status == 2 || ($this->payment_params->allow_zero_confirm == "1" && $status >= 0 && floatval($posted['received_amount']) >= floatval($posted['amount2']))) {
			$this->modifyOrder($order_id, $this->payment_params->verified_status, true, true); 
			$complete = true;
		}
		print "IPN OK\n";
		return $complete;
	}
}
